/**
 * Copyright (c) 2020, WSO2 Inc. (http://www.wso2.com). All Rights Reserved.
 *
 * This software is the property of WSO2 Inc. and its suppliers, if any.
 * Dissemination of any information or reproduction of any material contained
 * herein in any form is strictly forbidden, unless permitted by WSO2 expressly.
 * You may not alter or remove any copyright or other notice from copies of this content."
 */

import { TestableComponentInterface } from "@wso2is/core/models";
import { EmphasizedSegment, Heading, LabeledCard } from "@wso2is/react-components";
import React, { FunctionComponent, ReactElement, useEffect, useState } from "react";
import { useTranslation } from "react-i18next";
import { useSelector } from "react-redux";
import { Divider, Grid, Header, List, Segment } from "semantic-ui-react";
import {
    AppState,
    DocPanelUICardInterface,
    TechnologyLogos,
    PortalDocumentationStructureInterface
} from "../../../../features/core";
import {
    InboundProtocolListItemInterface,
    OIDCApplicationConfigurationInterface,
    SAMLApplicationConfigurationInterface,
    ApplicationManagementConstants,
    ApplicationManagementUtils
} from "../../../../features/applications";
import QuickStartIllustration from "../../../assets/images/illustrations/quick-start-illustration.svg";

/**
 * Proptypes for the applications help panel overview component.
 */
interface HelpPanelOverviewPropsInterface extends TestableComponentInterface {
    inboundProtocols?: InboundProtocolListItemInterface[];
    handleTabChange?: (tabId: number) => void;
    applicationType?: string;
    helpPanelDocStructure: PortalDocumentationStructureInterface;
    samples: DocPanelUICardInterface[];
    handleSelection: (sample) => void;
    onTOCClick?: (e: any, index: number) => void;
}

/**
 * Applications Help Panel Overview Component.
 *
 * @param {HelpPanelOverviewPropsInterface} props - Props injected to the component.
 *
 * @return {React.ReactElement}
 */
export const HelpPanelOverview: FunctionComponent<HelpPanelOverviewPropsInterface> = (
    props: HelpPanelOverviewPropsInterface
): ReactElement => {

    const urlSearchParams = new URLSearchParams(location.search);

    const oidcConfigurations: OIDCApplicationConfigurationInterface = useSelector(
        (state: AppState) => state.application.oidcConfigurations);
    const samlConfigurations: SAMLApplicationConfigurationInterface = useSelector(
        (state: AppState) => state.application.samlConfigurations);
    const { t } = useTranslation();

    const { applicationType, inboundProtocols, samples, handleSelection, onTOCClick } = props;

    const [ isOIDC, setIsOIDC ] = useState<boolean>(false);
    const [ isSAML, setIsSAML ] = useState<boolean>(false);
    const [ isOIDCConfigsLoading, setOIDCConfigsLoading ] = useState<boolean>(false);
    const [ isSAMLConfigsLoading, setSAMLConfigsLoading ] = useState<boolean>(false);
    const [ helpPanelSelectedSample, setHelpPanelSelectedSample ] = useState<DocPanelUICardInterface>(undefined);

    useEffect(() => {
        if (inboundProtocols == undefined) {
            return;
        }

        if (inboundProtocols.length > 0) {
            inboundProtocols.map((protocol) => {
                if (protocol.type == "oauth2") {
                    setIsOIDC(true);
                } else if (protocol.type == "samlsso") {
                    setIsSAML(true);
                }
            })
        }

    }, [ inboundProtocols ]);

    useEffect(() => {
        if (oidcConfigurations !== undefined) {
            return;
        }

        ApplicationManagementUtils.getOIDCApplicationMeta()
            .finally(() => {
                setOIDCConfigsLoading(false);
            });
    }, [ oidcConfigurations ]);

    useEffect(() => {
        if (samlConfigurations !== undefined) {
            return;
        }

        ApplicationManagementUtils.getSAMLApplicationMeta()
            .finally(() => {
                setSAMLConfigsLoading(false);
            });
    }, [ samlConfigurations ]);

    return (
        <>
            <Grid>
                {
                    (urlSearchParams.get(ApplicationManagementConstants.APP_STATE_URL_SEARCH_PARAM_KEY)
                        === ApplicationManagementConstants.APP_STATE_URL_SEARCH_PARAM_VALUE) &&
                    <Grid.Row>
                        <Grid.Column width={ 16 }>
                            { /** TODO : Need to move inline styles to sperate file */}
                            <EmphasizedSegment padded style={ {
                                backgroundImage: `url(${ QuickStartIllustration })`,
                                backgroundRepeat: "no-repeat",
                                backgroundSize: "contain",
                                backgroundPosition: "right",
                                backgroundColor: "#fff",
                                backgroundOrigin: "content-box",
                                padding: "2em"
                            } }>
                                <Heading as="h3">
                                    You&apos;re few steps away from completing the integration!
                                </Heading>
                                <Header.Subheader
                                    style={ { fontWeight: 200, marginRight: "16em" } }>
                                    An application has been configured for you with the recommended
                                    settings.
                                    Please select one of the following methods to get started.
                                </Header.Subheader>
                                <Divider hidden />
                                <Divider hidden />
                                <Divider hidden />
                                <Divider hidden />
                            </EmphasizedSegment>
                        </Grid.Column>
                    </Grid.Row>
                }
                {
                    applicationType && ( applicationType == ApplicationManagementConstants.SPA 
                        || applicationType == "Web Application" )
                        ? (
                            <>
                                <Grid.Row>
                                    <Grid.Column width={ 16 } >
                                        { /** TODO : Need to move inline styles to sperate file */}
                                        <Segment 
                                            style={ { "background": "#fff", "box-shadow": "none", "width": "100%" } } 
                                            placeholder compact padded
                                        >
                                            <Grid columns={ 2 } stackable>
                                                <Divider vertical>Or</Divider>
                                                <Grid.Row>
                                                    <Grid.Column>
                                                        <Heading style={ { "marginBottom":"0.2em" } } as="h3">
                                                            Try out with a sample
                                                        </Heading>
                                                        <p style={ { fontWeight: 200 } }>
                                                            Use our pre configured samples and try out the
                                                            authentication.
                                                        </p>
                                                        <Divider hidden />
                                                        <div>
                                                            {
                                                                samples && samples.map((sample, index) => (
                                                                    <LabeledCard
                                                                        key={ index }
                                                                        size="tiny"
                                                                        label={ sample.displayName }
                                                                        image={ 
                                                                            TechnologyLogos[ sample.image ] 
                                                                        }
                                                                        imageSize="mini"
                                                                        onClick={ () =>
                                                                            handleSelection(sample)
                                                                        }
                                                                    />
                                                                ))
                                                            }
                                                        </div>
                                                    </Grid.Column>
                                                    <Grid.Column>
                                                        { /** TODO : Need to move inline styles to sperate file */}
                                                        <div style={ { paddingLeft:"1.5em" } }>
                                                            <Heading style={ { "marginBottom":"0.2em" } } as="h3">
                                                                Integrate with your application
                                                            </Heading>
                                                            <p style={ { fontWeight: 200 } }>
                                                                Follow the below steps to integrate your own
                                                                application.
                                                            </p>
                                                            <Divider hidden/>
                                                            <Divider hidden/>
                                                            <ol style={ { paddingLeft: "1em", fontWeight: 200 } }>
                                                                <li
                                                                    className="pb-1 link pointing"
                                                                    onClick={ (e) => onTOCClick(e, 1) }
                                                                >
                                                                    <a>Configure Application Settings</a>
                                                                </li>
                                                                <li
                                                                    className="pb-1 link pointing"
                                                                    onClick={ (e) => onTOCClick(e, 2) }
                                                                >
                                                                    <a>Install and configure the SDK</a>
                                                                </li>
                                                                <li
                                                                    className="pb-1 link pointing"
                                                                    onClick={ (e) => onTOCClick(e, 3) }
                                                                >
                                                                    <a>Ensuring Application Security</a>
                                                                </li>
                                                                <li
                                                                    className="pb-1 link pointing"
                                                                    onClick={ (e) => onTOCClick(e, 4) }
                                                                >
                                                                    <a>Further Reading</a>
                                                                </li>
                                                            </ol>
                                                        </div>
                                                    </Grid.Column>
                                                </Grid.Row>
                                            </Grid>
                                        </Segment>
                                    </Grid.Column>
                                </Grid.Row>
                            </>
                        )
                        : null
                }
            </Grid>
        </>
    );
};
